/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package okototentest;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;

/**
 *
 * @author kosen
 */
public abstract class OkototenText {
    public static final int MASK_WARITYU = 0b0001;
    public static final int MASK_KAIGYO = 0b0010;
    
    //このクラスにおいては文字を指定する固有のIDはすべてPoint型とする。
    //Point型はテキストファイル中の文字の論理的な登場位置を表す
    //符号、タグなどにより本文の文字以外にも座標がふられるが、表示系でこの問題は処理してもらう
    
    //入力されていた文字そのもののデータ
    protected HashMap<OkotoPlace,String> mojiSet;
    //各文字に付与されているヲコト点データ
    protected HashMap<OkotoPlace,ArrayList<OkototenElement>> dataset;
    //元の行につけられた名前（元データ）
    protected ArrayList<String> names;
    //元の行のテキスト（元データ）
    protected ArrayList<String> text;
    //１行の最大文字数
    protected int maxRowCount = -1;
    //割注情報
    protected HashMap<OkotoPlace,Integer> warityu;
    //語順点の情報
    protected HashMap<OkotoPlace,ArrayList<GojuntenElement>> gojundataset;
    //仮名点の情報
    protected HashMap<OkotoPlace,ArrayList<KanaElement>> kanadataset;
    //コメントの情報
    protected HashMap<OkotoPlace,String> commentset;
    
    
    
    public OkototenText(){
        this.mojiSet = new HashMap<OkotoPlace,String>();
        this.dataset = new HashMap<OkotoPlace,ArrayList<OkototenElement>>();
        this.gojundataset = new HashMap<OkotoPlace,ArrayList<GojuntenElement>>();
        this.kanadataset = new HashMap<OkotoPlace,ArrayList<KanaElement>>();
        this.commentset = new HashMap<OkotoPlace,String>();
        this.warityu = new HashMap<OkotoPlace,Integer>();
        
        this.names = new ArrayList<String>();
        this.text = new ArrayList<String>();
    }

    public OkototenText(TaggedTextParser parser,File datafile,String encode) throws IOException{
        this.mojiSet = new HashMap<OkotoPlace,String>();
        this.dataset = new HashMap<OkotoPlace,ArrayList<OkototenElement>>();
        this.gojundataset = new HashMap<OkotoPlace,ArrayList<GojuntenElement>>();
        this.kanadataset = new HashMap<OkotoPlace,ArrayList<KanaElement>>();
        this.commentset = new HashMap<OkotoPlace,String>();
        this.warityu = new HashMap<OkotoPlace,Integer>();
        this.maxRowCount = parser.readFile(datafile, encode);
        this.names = parser.getIndexNames();
        this.text = parser.getInputText();
        for(int line=0;line<this.text.size();line++){
            String lineText = this.text.get(line);
            int specalCharCount = 0;
            boolean inWariTyu = false;
            for(int column=0;column<lineText.length();column++){
                String targetChar = lineText.substring(column, column+1);
                //割注を表す特殊文字への対応
                if(targetChar.equals("［")){
                    inWariTyu = true;
                    specalCharCount++;
                } else if(targetChar.equals("］")){
                    inWariTyu = false;
                    specalCharCount++;
                } else if(targetChar.equals("／")){
                    specalCharCount++;
                } else {
                    OkotoPlace place = new OkotoPlace(line+1,column+1-specalCharCount);
                    //割注の場合はこれが／前の最後の文字かも調べておく
                    int warimask = 0;
                    if(inWariTyu){
                        warimask = warimask | MASK_WARITYU;
                        if(lineText.substring(column+1, column+2).equals("／")){
                            warimask = warimask | MASK_KAIGYO;
                        }
                        this.warityu.put(place, warimask);
                    }
                    //単なる文字
                    this.mojiSet.put(place, targetChar);
                    this.dataset.put(place, new ArrayList<OkototenElement>());
                }
            }
        }
    }

    
    public ArrayList<OkotoPlace> getMojiKeyset(){
        
        ArrayList<OkotoPlace> keyset = new ArrayList<OkotoPlace>(mojiSet.keySet());
        Collections.sort(keyset, new Comparator<OkotoPlace>(){
            public int compare(OkotoPlace t1, OkotoPlace t2) {
                if(t1.getLineNumber() == t2.getLineNumber()){
                    return Integer.compare(t1.getColumnNumber(),t2.getColumnNumber());
                } else {
                    return Integer.compare(t1.getLineNumber(),t2.getLineNumber());
                }
            }
        });

        return keyset;
    }    
    public HashMap<OkotoPlace, String> getMojiSet() {
        return this.mojiSet;
    }

    public HashMap<OkotoPlace, ArrayList<OkototenElement>> getDataSet() {
        return this.dataset;
    }

    public HashMap<OkotoPlace, ArrayList<GojuntenElement>> getGojuntenDataSet() {
        return this.gojundataset;
    }

    public HashMap<OkotoPlace, String> getCommentDataSet() {
        return this.commentset;
    }

    
    public HashMap<OkotoPlace,ArrayList<KanaElement>> getKanaDataSet(){
        return this.kanadataset;
    }
    public void setMojiData(OkotoPlace trancePoint, String string) {
        this.mojiSet.put(trancePoint, string);
    }
    
    public void addOkotoTenElement(OkotoPlace trancePoint, OkototenElement ten) {
        ArrayList<OkototenElement> currentdata = this.dataset.get(trancePoint);
        currentdata.add(ten);
        this.dataset.put(trancePoint, currentdata);
    }

    public void removeOkotoTenElement(OkotoPlace trancePoint, OkototenElement ten) {
        ArrayList<OkototenElement> currentdata = this.dataset.get(trancePoint);
        currentdata.remove(ten);
        this.dataset.put(trancePoint, currentdata);
    }

    public void setOkotoTenElements(OkotoPlace trancePoint, ArrayList<OkototenElement> tens) {
        this.dataset.put(trancePoint, tens);
    }
    public void setGojunTenElements(OkotoPlace trancePoint, ArrayList<GojuntenElement> tens) {
        this.gojundataset.put(trancePoint, tens);
    }
    public void setKanaElements(OkotoPlace trancePoint, ArrayList<KanaElement> tens) {
        this.kanadataset.put(trancePoint, tens);
    }
    public void setComment(OkotoPlace trancePoint,String comment){
        this.commentset.put(trancePoint, comment);
    }
    
       
    /** 行につけられた名前を返す */
    public ArrayList<String> getIndexNames() {
        return this.names;
    }


    /** 行の文字データを返す */
    public ArrayList<String> getInputText() {
        return this.text;
    }
    
    /** ヲコト点が付与されているかを確認するためのメソッド：セルの塗り分けに使う */
    public boolean hasOkotoElement(OkotoPlace place) {
        if(this.dataset.containsKey(place)){
            ArrayList<OkototenElement> list = this.dataset.get(place);
            if(list.size() > 0){
                return true;
            }
        }
        return false;
    }
    
    /** 割注かどうかの情報を返す */
    public int getWarityuMask(OkotoPlace place){
        if(this.warityu.containsKey(place)){
            return this.warityu.get(place);
        }
        return 0;
    }
    
    public boolean hasGojunElement(OkotoPlace place) {
        if(this.gojundataset.containsKey(place)){
            return true;
        }
        return false;
    }
    
    public boolean hasKanaElement(OkotoPlace place){
        if(this.kanadataset.containsKey(place)){
            return true;
        }
        return false;
    }
     public boolean hasComment(OkotoPlace place){
        if(this.commentset.containsKey(place)){
            return true;
        }
        return false;
    }
       
    /** 複数文字にまたがる仮名点を取得 */
    public ArrayList<KanaElement> getMultiKanaElement(OkotoPlace place){
        ArrayList<KanaElement> list = new ArrayList<KanaElement>();
        if(this.kanadataset.containsKey(place)){
            for(KanaElement e:this.kanadataset.get(place)){
                if(e.targetLength > 1){
                    list.add(e);
                }
            }
        }
        return list;
    }
    
    public abstract boolean openFile(File file,String encode) throws IOException;
    public abstract boolean saveFile(File file,String encode) throws IOException;
}
