/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package okototentest.gui;

import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.net.URL;
import java.util.HashMap;
import javax.imageio.ImageIO;
import javax.swing.ImageIcon;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import utf16.UnicodeCharactor;

/**
 *
 * @author t-koji
 */
public class CharprintedPanel extends JPanel implements ClickableOkotoMoji{
    private String printChar = " ";
    private int gridSize = 7;    
    private BufferedImage myImage;
    
    private int gridWidth;
    
    private HashMap<Point,Color> fillPointsColor;
    private Color cellFillColorRed = new Color(255,0,0,100);
    private Color cellFillColorBlack = new Color(0,0,0,100);
    private Color cellFillColorBlue = new Color(0,0,255,100);
    private Color cellFillColorGreen = new Color(0,255,0,100);

    public CharprintedPanel(){
        this.fillPointsColor = new HashMap<Point,Color>();
        
    }
    
    public void setGridSize(int size){
        this.gridSize = size;
    }
    
    public void setImage(BufferedImage img){
        this.myImage = img;
    }
    
    public void setImageChar(String s){
        if(s.length() > 0){
            this.printChar = s.substring(0,1);
            this.myImage = new BufferedImage(this.getPreferredSize().width,this.getPreferredSize().height,BufferedImage.TYPE_INT_RGB);
            
            Graphics2D g = this.myImage.createGraphics();
            g.setColor(Color.WHITE);
            g.fillRect(0, 0, this.getPreferredSize().width, this.getPreferredSize().height);
            g.setColor(Color.black);
            //Font f = new Font(this.getFont().getFontName(Locale.JAPANESE),Font.PLAIN,this.getPreferredSize().width);

                
            //Font f = new Font("IPA明朝",Font.PLAIN,this.getPreferredSize().width);
            Font f = main.bootstrap.CharFont.deriveFont(Font.PLAIN,this.getPreferredSize().width);
            g.setFont(f);
            g.drawString(this.printChar, 0,this.myImage.getHeight()-40);
//          this.viewCheck();
            this.validate();
            this.repaint();
        }
    }
    
    public void viewCheck(){
        JDialog dialog = new JDialog();
        dialog.setSize(this.getPreferredSize().width+30, this.getPreferredSize().height+50);
        JLabel testLabel = new JLabel(new ImageIcon(this.myImage));
        dialog.add(testLabel);
        dialog.pack();
        dialog.setVisible(true);
    }
    
    @Override
    public void paint(Graphics g) {
        //super.paint(g);
        if(this.myImage != null){
            this.gridWidth = (this.getPreferredSize().width) / this.gridSize;
            
            
            int start = gridWidth;
            int imageWidth = gridWidth*(gridSize-2);
            //文字を書く
            g.setColor(Color.black);
            g.drawImage(myImage, start+gridWidth/2, start+gridWidth/2, imageWidth-gridWidth, imageWidth-gridWidth, this);

            
            //面に色を塗る
            for(Point p:this.fillPointsColor.keySet()){
                g.setColor(this.fillPointsColor.get(p));
                int[] fillarea = getFillArea(p,gridWidth);
                g.fillRect(fillarea[0], fillarea[1], fillarea[2], fillarea[3]);
            }

            //線枠を書く
            g.setColor(Color.black);
            for(int i=0;i<this.gridSize+1;i++){
                g.drawLine(0,gridWidth*i, this.getPreferredSize().width, gridWidth*i);
                g.drawLine(gridWidth*i, 0,gridWidth*i,this.getPreferredSize().width);
            }
            
            g.setColor(Color.red);
            g.drawLine(start+gridWidth/2, start+gridWidth/2, start+gridWidth/2, start+imageWidth-gridWidth/2);
            g.drawLine(start+gridWidth/2, start+imageWidth-gridWidth/2, start+imageWidth-gridWidth/2, start+imageWidth-gridWidth/2);
            g.drawLine(start+imageWidth-gridWidth/2, start+imageWidth-gridWidth/2, start+imageWidth-gridWidth/2, start+gridWidth/2);
            g.drawLine(start+imageWidth-gridWidth/2, start+gridWidth/2, start+gridWidth/2, start+gridWidth/2);

            
            g.drawLine(start+gridWidth/2*5, start+gridWidth/2, start+gridWidth/2*5, start+imageWidth-gridWidth/2);
            g.drawLine(start+imageWidth-gridWidth/2, start+gridWidth/2*5, start+gridWidth/2, start+gridWidth/2*5);
            
            //座標情報を追記
            g.setColor(Color.blue);
            for(int x=-3;x<=3;x++){
                g.drawString(String.valueOf(x), gridWidth/2 + (x+3)*gridWidth - 5, gridWidth/2);
            }
            for(int y=-2;y<=3;y++){
                g.drawString(String.valueOf(y), gridWidth/2 - 5, gridWidth/2 + (y+3)*gridWidth);
            }
            
            
        }
    }

    private int[] getFillArea(Point p,int gridWidth){
        int[] fillarea = new int[4];
        
        fillarea[0] = (p.x+3)*gridWidth;//塗り始めのX座標
        fillarea[1] = (p.y+3)*gridWidth;//塗り始めのY座標
        fillarea[2] = gridWidth;//塗幅
        fillarea[3] = gridWidth;//塗高さ

        return fillarea;
    }
    
    /** マウスがクリックされた座標からヲコト点ポイントを計算 */
    public Point getClickedLocation(Point mousePoint) {
        return new Point(mousePoint.x/this.gridWidth,mousePoint.y/this.gridWidth);
    }
    
    //色を塗るセルの追加
    public void addFillCell(Point p,String color){
        if(!this.fillPointsColor.containsKey(p)){
            Color fillcolor;
            if(color.equals("朱")){
                fillcolor = this.cellFillColorRed;
            } else if(color.equals("墨")){
                fillcolor = this.cellFillColorBlack;
            } else if(color.equals("角")){
                fillcolor = this.cellFillColorGreen;
            } else {
                fillcolor = this.cellFillColorBlue;
            }
            this.fillPointsColor.put(p, fillcolor);
        }
        this.validate();
    }

    //色を塗るセルの削除
    public void removeFillCell(Point p){
        if(this.fillPointsColor.containsKey(p)){
            this.fillPointsColor.remove(p);
        }
        this.validate();
        
    }
    
}
