package utf16;


import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintStream;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

//更新履歴
//2009.07.27  文字列からUnicode配列を作る際にサロゲートを削除する処理を追加
//2009.04.27　ファイルクラスからの読み込みを許可
//2009.03.26　最終行を読み込まないバグを修正
//2007.08.16　ストリーム処理を独立化
//2007.07.31　例外処理、叱文字対応の追加
//2007.06.04　例外処理 UTF16変換エラーを追加
//2007.02.17　作成
public class UnicodeCharactor {
	//例外文字の置換を行うか　「(叱)」　→　「U+20B9F」（１文字）に置換など。
	public static final boolean replaceExceptionChar = false;

	public static boolean isRightUnicode(String Unicode){
		if(Unicode.startsWith("U+")){
			Unicode = Unicode.substring(2);
		}
		try{
			int code = Integer.parseInt(Unicode,16);
			return Character.isDefined(code);
		} catch(NumberFormatException e){
			return false;
		}
	}
	
	public static String decodeUnicode(String Unicode){
		if(Unicode.startsWith("U+")){
			Unicode = Unicode.substring(2);
		}
		try{
			int code = Integer.parseInt(Unicode,16);
			if(replaceExceptionChar){
				//特殊置換対策用(叱)
				if(code == 0x20B9F){
					return "("+new String(Character.toChars(0x53F1))+")";
				}
			}
			return new String(Character.toChars(code));
		} catch(NumberFormatException e){
			return null;
		}
	}
	
	public static String toUnicodePattern(String string){
		if(string.startsWith("U+")){
			string = string.substring(2);
		}
		try{
			int codePoint = Integer.parseInt(string,16);
			return toUnicodePattern(codePoint);
		} catch(NumberFormatException e){
			return null;
		}
	}
	
	public static String toUnicodePattern(int codePoint){
		//文字コード生成
		StringBuilder codebuf = new StringBuilder();
		codebuf.append(Integer.toHexString(codePoint));
		while(codebuf.length() < 4){
			codebuf.insert(0,"0");
		}
		codebuf.insert(0,"U+");
		return codebuf.toString().toUpperCase();
	}
	
	public static DecimalFormat ByteFormat = new DecimalFormat("00000000");
	
	/** ストリームから読み込んでUTF-16LEにエンコードする 
	 * @throws IOException 
	 * @throws UTF16FormatException */
	public static String readStream(InputStream input) throws IOException, UTF16FormatException{
		int[] readbyte = new int[2];
		//２バイト読み込む
		for(int i=0;i<2;i++){
			int readdata = input.read();
			if(readdata == -1){
				return null;
			}
			readbyte[i] = readdata;
		}
			
		String twoBytesBinary = "";
		for(int i=0;i<2;i++){
			if(readbyte[i] != -1){
				String binaryString = ByteFormat.format(Integer.parseInt(Integer.toBinaryString(readbyte[i])));
				twoBytesBinary = twoBytesBinary.concat(binaryString);
			}
		}
		if(!twoBytesBinary.equals("")){
			// 4バイト文字に対する処理
			if(twoBytesBinary.matches("........110110..")){
				//UTF-16LEの４バイト文字パターンの先頭２バイトにマッチしたらもう２バイト読み込む					
				for(int i=0;i<2;i++){
					int readdata = input.read();
					if(readdata == -1){
						System.err.println("4バイトで構成される文字が切れています。");
						throw new UTF16FormatException("4バイトで構成される文字が切れています。");
					}
					readbyte[i] = readdata;
				}
				String appendBinary ="";
				for(int i=0;i<2;i++){
					if(readbyte[i] != -1){
						String binaryString = ByteFormat.format(Integer.parseInt(Integer.toBinaryString(readbyte[i])));
						appendBinary = appendBinary.concat(binaryString);
					}
				}
				
				if(appendBinary.matches("........110111..")){
					//ここで４バイト構成可能
					return toFourByteUnicodeValue(twoBytesBinary,appendBinary);
				}else {
					//読み込んだもう２バイトにエラー
					System.err.println("4バイトで構成される文字が切れています。");
					throw new UTF16FormatException("4バイトで構成される文字が切れています。");
				}
			} else {
				//２バイト文字の場合
				return toTowByteUnicodeValue(twoBytesBinary);
			}
		}
		
		return null;
	}
	
	public static String unicodeToReferString(String unicode,boolean hexPattern) throws UTF16FormatException{
		if(UnicodeCharactor.isRightUnicode(unicode)){
			StringBuilder sbuf = new StringBuilder();
			int codePoint = Integer.parseInt(unicode.substring(2),16);
			if(hexPattern){
				sbuf.append("&#x");
				sbuf.append(Integer.toHexString(codePoint).toUpperCase());
			} else {
				sbuf.append("&#");
				sbuf.append(codePoint);
			}
			sbuf.append(";");
			return sbuf.toString();

		} else {
			throw new UTF16FormatException("文字参照へ変換する対象文字が正しいUnicode形式ではありませんでした。");
		}
	}
	
	private static Pattern ReferCharPattern16 = Pattern.compile("&#x([0-9A-Fa-f]+);");
	private static Pattern ReferCharPattern10 = Pattern.compile("&#([0-9]+);");
	public static String ReferStringToUnicode(String baseLine,boolean hexPattern){
		Pattern ReferCharPattern;
		if(hexPattern){
			ReferCharPattern = ReferCharPattern16;
		} else {
			ReferCharPattern = ReferCharPattern10;
		}

		StringBuffer buf = new StringBuffer(baseLine);
		//&#シリーズの置換
		Matcher mc = ReferCharPattern.matcher(buf);
		int start = 0;
		while(mc.find(start)){
			start = mc.start();
			String referCharCode = mc.group(1);
			int CodePoint = -1;
			if(hexPattern){
				CodePoint = Integer.parseInt(referCharCode,16);
			} else {
				CodePoint = Integer.parseInt(referCharCode,10);
			}

			String newChar = new String(Character.toChars(CodePoint));
			buf.replace(start,mc.end(),newChar);
			
			start = start + newChar.length();
			mc = ReferCharPattern.matcher(buf);
		}
		return buf.toString();
	}
	
	public static String toTowByteUnicodeValue(String binaryString){
		StringBuffer UniCodePattern = new StringBuffer();
		UniCodePattern.append("U+");

		char[] chararray = binaryString.toCharArray();
		StringBuffer[] code = new StringBuffer[4];
		code[0] = new StringBuffer(4);
		code[0].append(chararray[8]);
		code[0].append(chararray[9]);
		code[0].append(chararray[10]);
		code[0].append(chararray[11]);
		code[1] = new StringBuffer(4);
		code[1].append(chararray[12]);
		code[1].append(chararray[13]);
		code[1].append(chararray[14]);
		code[1].append(chararray[15]);
		code[2] = new StringBuffer(4);
		code[2].append(chararray[0]);
		code[2].append(chararray[1]);
		code[2].append(chararray[2]);
		code[2].append(chararray[3]);
		code[3] = new StringBuffer(4);
		code[3].append(chararray[4]);
		code[3].append(chararray[5]);
		code[3].append(chararray[6]);
		code[3].append(chararray[7]);
		UniCodePattern.append(Integer.toHexString(Integer.parseInt(code[0].toString(),2)).toUpperCase());
		UniCodePattern.append(Integer.toHexString(Integer.parseInt(code[1].toString(),2)).toUpperCase());
		UniCodePattern.append(Integer.toHexString(Integer.parseInt(code[2].toString(),2)).toUpperCase());
		UniCodePattern.append(Integer.toHexString(Integer.parseInt(code[3].toString(),2)).toUpperCase());
		
		return UniCodePattern.toString();
	}
	
	public static String toFourByteUnicodeValue(String binaryString,String appendBinary){
		StringBuffer UniCodePattern = new StringBuffer();
		UniCodePattern.append("U+");

		//４バイト文字のもうひとつも手に入ったのでコード作成可能
		char[] chararray = (binaryString+appendBinary).toCharArray();
		
		StringBuffer wwww = new StringBuffer();
		wwww.append(chararray[14]);
		wwww.append(chararray[15]);
		wwww.append(chararray[0]);
		wwww.append(chararray[1]);
		
		StringBuffer[] code = new StringBuffer[4];
		code[0] = new StringBuffer(4);
		code[0].append(chararray[2]);
		code[0].append(chararray[3]);
		code[0].append(chararray[4]);
		code[0].append(chararray[5]);
		code[1] = new StringBuffer(4);
		code[1].append(chararray[6]);
		code[1].append(chararray[7]);
		code[1].append(chararray[30]);
		code[1].append(chararray[31]);
		code[2] = new StringBuffer(4);
		code[2].append(chararray[16]);
		code[2].append(chararray[17]);
		code[2].append(chararray[18]);
		code[2].append(chararray[19]);
		code[3] = new StringBuffer(4);
		code[3].append(chararray[20]);
		code[3].append(chararray[21]);
		code[3].append(chararray[22]);
		code[3].append(chararray[23]);
		

		UniCodePattern.append(Integer.toHexString(Integer.parseInt(wwww.toString(),2) + 1).toUpperCase());
		UniCodePattern.append(Integer.toHexString(Integer.parseInt(code[0].toString(),2)).toUpperCase());
		UniCodePattern.append(Integer.toHexString(Integer.parseInt(code[1].toString(),2)).toUpperCase());
		UniCodePattern.append(Integer.toHexString(Integer.parseInt(code[2].toString(),2)).toUpperCase());
		UniCodePattern.append(Integer.toHexString(Integer.parseInt(code[3].toString(),2)).toUpperCase());

		return UniCodePattern.toString();
	}
	
	public static ArrayList<ArrayList<String>> readFileByUnicode(File file) throws UTF16FormatException{
		return readFileByUnicode(file.getPath());
	}
	public static ArrayList<ArrayList<String>> readFileByUnicode(String Filename) throws UTF16FormatException{
		
		ArrayList<ArrayList<String>> DataTable = new ArrayList<ArrayList<String>>();
		ArrayList<String> readingLine = new ArrayList<String>(); //読み込み中の行

		try {
			InputStream reader = new BufferedInputStream(new FileInputStream(Filename));

			String UniCodePattern;
			while((UniCodePattern = readStream(reader)) != null){
				//一文字づつ読み込む
				// 一文字読み込む処理がここまでで終了。
				// UniCodePattern　に読み込んだ文字が格納されている。
				
				// CRだけの場合は新しい行にしない。次文字で判断する
				if(readingLine.size() != 0){
					String LastChar = readingLine.get(readingLine.size() - 1);
					if(LastChar.equals("U+000D")){//前文字がCRの場合
						//次にLF以外の文字なら新しい行になる。
						if(!UniCodePattern.toString().equals("U+000A")){
							//例外文字の修正
							if(replaceExceptionChar){
								readingLine = repalce(readingLine);
							}
							DataTable.add(readingLine);
							readingLine = new ArrayList<String>();
						}
					}
				}

				//文字を現在の行に追加
				readingLine.add(UniCodePattern.toString());
				
				
				// 追加した文字がLFなら、次からは新しい新しい行にする。
				if(UniCodePattern.toString().equals("U+000A")){
					if(replaceExceptionChar){
						readingLine = repalce(readingLine);
					}
					DataTable.add(readingLine);
					readingLine = new ArrayList<String>();
				}
				
				//一文字に対する処理ここまで
			}
			//最後の行がLFでなく行がデータに追加されて無い場合に対応
			if(readingLine.size() != 0){
				DataTable.add(readingLine);
			}
			reader.close();
		} catch (FileNotFoundException e) {
			System.err.println("ファイルが見つかりません。");
			e.printStackTrace();
		} catch (IOException e) {
			System.err.println("バイト変換エラー");
			e.printStackTrace();
		}
		return DataTable;
	}
	/** 特殊文字の置換
	 * 現在の対象は　叱　だけ */
	private static ArrayList<String> repalce(ArrayList<String> readingLine) {
		if(readingLine.contains("U+53F1")){
			ArrayList<String> newLine = new ArrayList<String>();
			for(int i=0;i<readingLine.size();i++){
				if(readingLine.get(i).equals("U+53F1")){
					if(i>0 && i<readingLine.size()-1){
						if(readingLine.get(i-1).equals("U+0028") && readingLine.get(i+1).equals("U+0029")){
							//左右が半角丸括弧で囲まれている場合
							newLine.remove(newLine.size() - 1);//前の括弧部分を消去
							newLine.add("U+20B9F");
							i++;//後ろの括弧は処理しない
						} else {
							newLine.add(readingLine.get(i));
						}
					} else { 
						newLine.add(readingLine.get(i));
					}
				} else {
					newLine.add(readingLine.get(i));
				}
			}
			return newLine;
		} else {
			return readingLine;
		}
	}
	
	public static ArrayList<String> StringToUnicodeArray(String Line){
		ArrayList<String> array = new ArrayList<String>();
		for(int i=0;i<Line.length();i++){
			int c = Line.codePointAt(i);
			if (c >= 0xD800 && c <= 0xDFFF){
				//サロゲート
			} else {
				array.add(toUnicodePattern(c));
			}
		}
		return array;
	}
	
	public static String arrayToString(ArrayList<String> array){
		StringBuffer buf = new StringBuffer();
		Iterator<String> iter = array.iterator();
		while(iter.hasNext()){
			String code = iter.next();
			
			if(!code.equals("U+FEFF") && !code.equals("U+000A") && !code.equals("U+000D")){
				String ch = UnicodeCharactor.decodeUnicode(code);
				if(ch != null){
					buf.append(ch);
				}
			}
			
			
		}
		return buf.toString();
	}
	
	public static String decode(String code){
		String s = null;
		int value;
		
		value = Integer.valueOf(code.substring(2),16);
		if(Character.isValidCodePoint(value)){
			s = new String(Character.toChars(value));
		}
		return s;
	}
	
	/** ユニコード文字一覧の保存 */
	public static void makeUnicodeListfile(ArrayList<ArrayList<String>> data,File writeFile){
		PrintStream writer = null;
		try {
			writer = new PrintStream(new FileOutputStream(writeFile));
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		
		
		Iterator<ArrayList<String>> iter = data.iterator();
		while(iter.hasNext()){
			ArrayList<String> Line = iter.next();
			Iterator<String> chariter = Line.iterator();
			while(chariter.hasNext()){
				writer.print(chariter.next());
				writer.print(" ");
			}
			writer.println();
		}
		writer.close();
	}

	
}
